@@ -6,101 +6,28 @@ use 5.008003;
 use Win32 ();
 use Win32::API ();
 use Carp ();
-use File::Spec::Functions qw/catfile splitdir/;
 use File::Basename qw/basename dirname/;
 use Exporter 'import';
 
+use Win32::Unicode::Util;
 use Win32::Unicode::Error;
-use Win32::Unicode::Encode;
 use Win32::Unicode::Constant;
+use Win32::Unicode::Define;
 use Win32::Unicode::File;
 use Win32::Unicode::Console;
 
 # export subs
-our @EXPORT    = qw/file_type file_size mkdirW rmdirW getcwdW chdirW findW finddepthW mkpathW rmtreeW mvtreeW cptreeW dir_size/;
+our @EXPORT    = qw/file_type file_size mkdirW rmdirW getcwdW chdirW findW finddepthW mkpathW rmtreeW mvtreeW cptreeW dir_size file_list dir_list/;
 our @EXPORT_OK = qw//;
 our %EXPORT_TAGS = ('all' => [@EXPORT, @EXPORT_OK]);
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 # global vars
 our $cwd;
 our $name;
 our $skip_pattern = qr/\A(?:(?:\.{1,2})|(?:System Volume Information))\z/;
 
-# FILETIME Struct
-Win32::API::Struct->typedef('FILETIME', qw(
-    DWORD dwLowDateTime;
-    DWORD dwHighDateTime;
-));
-
-# WIN32_FIND_DATAW Struct
-Win32::API::Struct->typedef('WIN32_FIND_DATAW', qw{
-    DWORD    dwFileAttributes;
-    FILETIME ftCreationTime;
-    FILETIME ftLastAccessTime;
-    FILETIME ftLastWriteTime;
-    DWORD    nFileSizeHigh;
-    DWORD    nFileSizeLow;
-    DWORD    dwReserved0;
-    DWORD    dwReserved1;
-    WCHAR    cFileName[520];
-    WCHAR    cAlternateFileName[28];
-});
-
-# GetCurrentDirectory
-my $GetCurrentDirectory = Win32::API->new('kernel32.dll',
-    'GetCurrentDirectoryW',
-    ['N', 'P'],
-    'N',
-) or die "GetCurrentDirectory: $^E";
-
-# SetCurrentDirectory
-my $SetCurrentDirectory = Win32::API->new('kernel32.dll',
-    'SetCurrentDirectoryW',
-    'P',
-    'I',
-) or die "SetCurrentDirectory: $^E";
-
-# FindFirstFile
-my $FindFirstFile = Win32::API->new('kernel32.dll',
-    'FindFirstFileW',
-    'PS',
-    'N',
-) or die "FindFirstFile: $^E";
-
-# FindNextFile
-my $FindNextFile = Win32::API->new('kernel32.dll',
-    'FindNextFileW',
-    'NS',
-    'I',
-) or die "FindNextFile $^E";
-
-# FindClose
-my $FindClose = Win32::API->new('kernel32.dll',
-    'FindClose',
-    'N',
-    'I',
-) or die "FileClose $^E";
-
-# GetcFileName
-my $GetcFileName = sub {
-    my $self = shift;
-    my $cFileName = do {
-        use bytes;
-        unpack "x44A520", $self->{FileInfo}->{buffer};
-    };
-    delete $self->{FileInfo}->{cFileName};
-    return utf16_to_utf8($cFileName . NULL);
-};
-
-# RemoveDirectory
-my $RemoveDirectory = Win32::API->new('kernel32.dll',
-    'RemoveDirectoryW',
-    'P',
-    'I',
-) or die "FileClose $^E";
-
 sub new {
     my $class = shift;
     bless {}, $class;
@@ -114,14 +41,15 @@ sub open {
     
     $self->{FileInfo} = Win32::API::Struct->new('WIN32_FIND_DATAW');
     
+    $dir = cygpathw($dir) or return if CYGWIN;
+    
     $self->{dir} = catfile $dir, '*';
     $dir = utf8_to_utf16($self->{dir}) . NULL;
     
-    $self->{handle} = $FindFirstFile->Call($dir, $self->{FileInfo});
+    $self->{handle} = FindFirstFile->Call($dir, $self->{FileInfo});
+    return Win32::Unicode::Error::_set_errno if $self->{handle} == INVALID_HANDLE_VALUE;
     
-    return 0 if $self->{handle} == INVALID_HANDLE_VALUE;
-    
-    $self->{first} = $GetcFileName->($self);
+    $self->{first} = $self->_cFileName;
     
     return $self;
 }
@@ -130,7 +58,7 @@ sub open {
 sub close {
     my $self = shift;
     _croakW("Can't open directory handle") unless $self->{handle};
-    return 0 unless $FindClose->Call($self->{handle});
+    return Win32::Unicode::Error::_set_errno unless FindClose->Call($self->{handle});
     delete @$self{qw[dir handle first FileInfo]};
     return 1;
 }
@@ -152,51 +80,63 @@ sub fetch {
         my @files;
         
         push @files, $first if $first;
-        while ($FindNextFile->Call($self->{handle} ,$self->{FileInfo})) {
-            push @files, $GetcFileName->($self);
+        while (FindNextFile->Call($self->{handle} ,$self->{FileInfo})) {
+            push @files, $self->_cFileName;
         }
         
         return @files;
     }
-    
     else {
         return $first if $first;
-        return unless $FindNextFile->Call($self->{handle} ,$self->{FileInfo});
-        return $GetcFileName->($self);
+        return Win32::Unicode::Error::_set_errno unless FindNextFile->Call($self->{handle} ,$self->{FileInfo});
+        return $self->_cFileName;
     }
 }
 
 *read = *readdir = \&fetch;
 
+sub _cFileName {
+    my $self = shift;
+    my $cFileName = do {
+        use bytes;
+        unpack "x44A520", $self->{FileInfo}->{buffer};
+    };
+    delete $self->{FileInfo}->{cFileName};
+    return utf16_to_utf8($cFileName . NULL);
+};
+
 # like use Cwd qw/getcwd/;
 sub getcwdW {
     my $buff = BUFF;
-    my $length = $GetCurrentDirectory->Call(MAX_PATH + 1, $buff);
+    my $length = GetCurrentDirectory->Call(MAX_PATH + 1, $buff);
     return utf16_to_utf8(substr $buff, 0, $length * 2);
 }
 
 # like CORE::chdir
 sub chdirW {
     my $set_dir = shift;
+    my $retry = shift || 0;
     _croakW('Usage: chdirW(dirname)') unless defined $set_dir;
+    $set_dir = cygpathw($set_dir) or return if CYGWIN;
     $set_dir = utf8_to_utf16(catfile $set_dir) . NULL;
-    return unless $SetCurrentDirectory->Call($set_dir);
+    return Win32::Unicode::Error::_set_errno unless SetCurrentDirectory->Call($set_dir);
+    return chdirW(utf16_to_utf8($set_dir), ++$retry) if CYGWIN && !$retry; # bug ?
     return 1;
 }
 
 # like CORE::mkdir
 sub mkdirW {
-    my $dir = shift;
-    _croakW('Usage: mkdirW(dirname)') unless defined $dir;
-    return Win32::CreateDirectory(catfile $dir) ? 1 : 0;
+    my $dir = defined $_[0] ? $_[0] : $_;
+    $dir = cygpathw($dir) or return if CYGWIN;
+    return Win32::CreateDirectory(catfile $dir) ? 1 : Win32::Unicode::Error::_set_errno;
 }
 
 # like CORE::rmdir
 sub rmdirW {
-    my $dir = shift;
-    _croakW('Usage: rmdirW(dirname)') unless defined $dir;
+    my $dir = defined $_[0] ? $_[0] : $_;
+    $dir = cygpathw($dir) or return if CYGWIN;
     $dir = utf8_to_utf16(catfile $dir) . NULL;
-    return $RemoveDirectory->Call($dir) ? 1 : 0;
+    return RemoveDirectory->Call($dir) ? 1 : Win32::Unicode::Error::_set_errno;
 }
 
 # like File::Path::rmtree
@@ -257,18 +197,21 @@ my $is_drive = qr/^[a-zA-Z]:/;
 my $in_dir   = qr#[\\/]$#;
 
 sub _cptree {
-    my $from = catfile shift;
+    my $from = shift;
     my $to = shift;
     my $over = shift || 0;
     my $bymove = shift || 0;
     
     _croakW("$from: no such directory") unless file_type d => $from;
     
+    $from = cygpathw($from) or return if CYGWIN;
+    $from = catfile $from;
+    $to = cygpathw($to) or return if CYGWIN;
+    
     if ($to =~ $is_drive) {
         $to = catfile $to, basename($from) if $to =~ $in_dir;
         $to = catfile $to;
     }
-    
     else {
         $to = catfile getcwdW(), $to, basename($from) if $to =~ $in_dir;
         $to = catfile getcwdW(), $to;
@@ -278,12 +221,12 @@ sub _cptree {
         mkpathW $to or _croakW("$to " . errorW);
     }
     
-    (my $replace_from = $from) =~ s/\\/\\\\/g;
+    my $replace_from = quotemeta $from;
     my $code = sub {
         my $from_file = $_;
         my $from_full_path = $Win32::Unicode::Dir::name;
         
-        (my $to_file = $from_full_path) =~ s/$replace_from\\?//;
+        (my $to_file = $from_full_path) =~ s/$replace_from//;
         $to_file = catfile $to, $to_file;
         
         if (file_type d => $from_file) {
@@ -331,8 +274,10 @@ sub _find_wrap {
     my $code = shift;
     my $bydepth = shift;
     for my $arg (@_) {
-        my $dir = catfile $arg;
-        _croakW("$dir: no such directory") unless file_type(d => $dir);
+        my $dir = $arg;
+        $dir = cygpathw($dir) or return if CYGWIN;
+        $dir = catfile $dir;
+       _croakW("$dir: no such directory") unless file_type(d => $dir);
         
         my $current = getcwdW;
         _find($code, $dir, $bydepth);
@@ -404,6 +349,28 @@ sub dir_size {
     return $size;
 }
 
+sub file_list {
+    my $dir = shift;
+    _croakW('Usage: file_list(dirname)') unless defined $dir;
+    
+    my $wdir = __PACKAGE__->new->open($dir) or return;
+    my @files = grep { !/^\.{1,2}$/ && file_type f => $_ } $wdir->fetch;
+    $wdir->close;
+    
+    return @files;
+}
+
+sub dir_list {
+    my $dir = shift;
+    _croakW('Usage: dir_list(dirname)') unless defined $dir;
+    
+    my $wdir = __PACKAGE__->new->open($dir) or return;
+    my @dirs = grep { !/^\.{1,2}$/ && file_type d => $_ } $wdir->fetch;
+    $wdir->close;
+    
+    return @dirs;
+}
+
 # return error message
 sub error {
     return errorW;
@@ -579,6 +546,18 @@ this function are slow.
 
   my $dir_size = dir_size($dir) or dieW errorW
 
+=item B<file_list($dir)>
+
+get files from $dir
+
+  my @files = file_list $dir;
+
+=item B<dir_list($dir)>
+
+get directorys from $dir
+
+  my @dirs = dir_list $dir;
+
 =back
 
 =head1 AUTHOR
@@ -592,7 +571,6 @@ L<Win32::API>
 L<Win32API::File>
 L<Win32::Unicode>
 L<Win32::Unicode::File>
-L<Win32::Unicode::Encode>
 L<Win32::Unicode::Error>
 
 =head1 LICENSE
